"""
    These are the sub-commands for a top level object added to the CLI group in 'cli.py'.
    The commands and options are implemented here and the logic behind them resides in the corresponding behavior file.
"""

from cli import pyke, behavior
import click

@click.group(help='Commands related to developing, managing and publishing experiences.')
def project():
  pass

@click.command('init', help="Initializes the current directory as a lumavate project")
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@pyke.auth.profile_context_required
def init(**kwargs):
  behavior.ProjectBehavior(**kwargs).init()

@click.command('get-qr', help="Display the QR activation code in the terminal")
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@pyke.auth.profile_context_required
def get_qr(**kwargs):
  behavior.ProjectBehavior(**kwargs).display_qr()



project.add_command(init)
project.add_command(get_qr)



"""
    The primary use of the ContainerUtil class is to connect to and communicate with
    the running docker daemon.
"""

from pathlib import Path
import docker
import shutil
import click
import gzip
import time
import sys
import os

class ContainerUtil:
  def __init__(self):
    try:
      self.client = docker.DockerClient(base_url='unix://var/run/docker.sock')
    except Exception as e:
      raise click.ClickException(click.style("Error communicating with the docker daemon. Are you sure it's running? {}".format(e), fg='red'))

  def save_image(self, image_name):
    try:
      image = self.client.images.get(image_name)
    except Exception as e:
      raise Exception(click.style('Error getting image: {}'.format(e), fg='red'))

    file_path = '{}/{}'.format(str(Path.home()), image_name.replace('/', '_').replace('.', '_').replace(':', '_'))

    with open(file_path, 'wb') as image_tar:
      for chunk in image.save(named=True):
        image_tar.write(chunk)

    zipped_file_path = self.zip_image(file_path)

    # Cleanup
    os.remove(file_path)

    return zipped_file_path

  def zip_image(self, file_path):
    with open(file_path, 'rb') as f_in:
      with gzip.open('{}.tar.gz'.format(file_path), 'w+b') as f_out:
        shutil.copyfileobj(f_in, f_out)

    return '{}.tar.gz'.format(file_path)




"""
    This behavior file contains the logic for a subset of commands. Logic specific to
    commands should be implamented in corresponding behavior files.
"""

from pathlib import Path
from cli import pyke, behavior
import click
import time
import json
import sys
import os

class WidgetVersionBehavior:
    def __init__(self, profile=None, widget=None, format=None, filter=None, page=None, pagesize=None, json=False):
        self.util = pyke.Util(profile=profile)
        if profile is not None:
            self.context = pyke.auth.load_cache()
        self.object_type = 'widget'
        self.json = json
        self.profile = profile
        self.widget = widget
        self.format = format
        self.filter = filter
        self.page = page
        self.pagesize = pagesize

    def list(self, widget):
        widget_id = self.util.lookup_object_id(self.object_type, widget)
        data = {'filter': self.filter, 'page': self.page, 'pagesize': self.pagesize}

        handlers = {
          'actualState': lambda x: click.style(x, fg='red') if x not in ['running'] else click.style(x, fg='green')
        }

        resp = self.util.cli_request('GET',
            self.util.build_url('{app}/iot/v1/containers/{widget_id}/versions?page={page}&pagesize={pagesize}&{filter}',
            {'widget_id': widget_id, **data}))

        if self.json:
            click.echo(json.dumps(resp))
            return

        self.util.print_table(resp['payload']['data'], self.format, handlers=handlers)
        self.util.print_record_count(resp)

    def start_stop(self, action, widget, version):
        widget_id = self.util.lookup_object_id(self.object_type, widget)
        version_data = None

        try:
            version_id = int(version)
        except:
            major, minor, patch = self.util.parse_version_number(version)

            version_data = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/containers/{widget_id}/versions?major={major}&minor={minor}&patch={patch}',\
                { 'widget_id': widget_id, 'major': major, 'minor': minor, 'patch': patch }))['payload']['data']

            if version_data is None or len(version_data) < 1:
                raise click.ClickException(click.style("Version not found", fg='red'))
            else:
                version_id = version_data[0].get('id')
                current_state = version_data[0].get('actualState')

        if version_data is None:
            version_data = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/containers/{widget_id}/versions/{version_id}',\
                {'widget_id': widget_id, 'version_id': version_id} ))['payload']['data']

            if version_data is None or len(version_data) < 1:
                raise click.ClickException(click.style("Version not found", fg='red'))

            current_state = version_data.get('actualState')

        if current_state not in ['stopped', 'running', 'error']:
            raise click.ClickException(click.style("To start or stop a version it must have a 'stopped', 'running' or 'error' state. The current state of this version is {}. \
                A version goes through several states after being uploaded to Lumavate. It must finish this process before this command can be run. \
                To check its current state, run 'luma widget-version ls'.".format(current_state), fg='yellow'))

        if action == 'stop' and current_state != 'running':
            raise click.ClickException(click.style("This version is not running", fg='yellow'))
        if action == 'start' and current_state == 'running':
            raise click.ClickException(click.style("This version is already running", fg='yellow'))

        titles = {
            'id': 'id',
            'actualState': 'Current State',
            'versionNumber': 'Version #',
            'createdAt': 'Created At',
            'createdBy': 'Created By'
        }

        data = {
            "action": action,
            "containerId": widget_id,
            "id": version_id
        }

        resp = self.util.cli_request('POST', self.util.build_url('{app}/iot/v1/containers/{widget_id}/versions/{version_id}/{action}',\
            {'widget_id': widget_id, 'version_id': version_id, 'action': action}), data=data)

        if self.json:
            click.echo(json.dumps(resp))
            return

        status_id = resp['payload']['data']['results']['statusId']
        progress = 0
        error = None
        if action == 'start':
            bar_name = "Starting widget"
        else:
            bar_name = "Stopping widget"

        with click.progressbar(length=100, label=bar_name) as bar:
            while progress < 100 and error is None:
                status_resp = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/statuses/{status_id}', {'status_id': status_id}))
                progress = status_resp['payload']['data']['overallPercent']
                error = status_resp['payload']['data']['errorMessage']
                bar.update(progress)

        click.echo(' ')
        ver_resp = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/containers/{widget_id}/versions/{version_id}', {'widget_id': widget_id, 'version_id': version_id}))

        handlers = {
          'actualState': lambda x: click.style(x, fg='red') if x not in ['running'] else click.style(x, fg='green')
        }

        self.util.print_table([ver_resp['payload']['data']], self.format, handlers=handlers, titles=titles)

    def run(self, widget, version, command):
        widget_id = self.util.lookup_object_id(self.object_type, widget)
        version_id = self.util.get_version_id(self.object_type, widget_id, version)

        data = {
            "command": command
        }

        resp = self.util.cli_request('POST', self.util.build_url('{app}/iot/v1/containers/{widget_id}/versions/{version_id}/exec',\
            {'widget_id': widget_id, 'version_id': version_id}), json=data)

        if self.json:
            click.echo(json.dumps(resp))
            return

        status_id = resp['payload']['data']['statusId']
        progress = 0
        error = None
        with click.progressbar(length=100, label='Executing command') as bar:
            while progress < 100:
                status_resp = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/statuses/{status_id}', {'status_id': status_id}))
                progress = status_resp['payload']['data']['overallPercent']
                bar.update(progress)
                if status_resp['payload']['data']['errorMessage'] is not None:
                    click.echo('There was an error running the command')
                    raise click.ClickException('Status: {}'.format(status_resp['payload']['data']))

        click.echo(' ')
        click.echo('Command Output: ')
        click.echo(' ')
        try:
            status_resp = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/statuses/{status_id}', {'status_id': status_id}))['payload']['data']
            summary = status_resp['summary']['results']
            if 'output' in summary[0].keys():
                if isinstance(summary[0]['output'], list):
                    for x in summary[0]['output']:
                        click.echo(x)
                else:
                    click.echo(summary)

        except:
            click.echo(status_resp)

    def logs(self, widget, version):
        widget_id = self.util.lookup_object_id(self.object_type, widget)
        version_id = self.util.get_version_id(self.object_type, widget_id, version)

        resp = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/containers/{widget_id}/versions/{version_id}/logs',\
            {'widget_id': widget_id, 'version_id': version_id}))

        if self.json:
            click.echo(json.dumps(resp))
            return

        for x in resp['payload']['data']:
            click.echo(x)

    def update(self, widget, version, label, format):
        widget_id = self.util.lookup_object_id(self.object_type, widget)
        version_id = self.util.get_version_id(self.object_type, widget_id, version)

        post_data = {
          'widgetId': widget_id,
          'id': version_id,
          'label': label
        }

        resp = self.util.cli_request('PUT',\
            self.util.build_url('{app}/iot/v1/containers/{widget_id}/versions/{version_id}',
            {'widget_id': widget_id, 'version_id': version_id}), data=json.dumps(post_data))

        if self.json:
            click.echo(json.dumps(resp))
            return

        self.util.print_table([resp['payload']['data']], format)

    def delete(self, widget, version, version_mask, format):
        if version is None and version_mask is None:
            version = str(input("Version: "))
            if version is None or version == '':
                raise click.BadParameter(click.style('You must provide either a --version or a --version-mask', fg='red'))

        widget_id = self.util.lookup_object_id(self.object_type, widget)

        if version is not None:
            version_id = self.util.get_version_id(self.object_type, widget_id, version)

            resp = self.util.cli_request('DELETE',\
                self.util.build_url('{app}/iot/v1/containers/{widget_id}/versions/{version_id}', {'widget_id': widget_id, 'version_id': version_id}))

            if self.json:
                click.echo(json.dumps(resp))
                return

            self.util.print_table([resp['payload']['data']], format)

        else:
            versions = self.util.get_versions_from_mask(self.object_type, widget_id, version_mask)

            if self.json:
                resp_list = {}
                resp_list['responses'] = []
                for v in versions:
                    resp = self.util.cli_request('DELETE',
                        self.util.build_url('{app}/iot/v1/containers/{widget_id}/versions/{version_id}', {'widget_id': widget_id, 'version_id': v['id']}))
                    resp_list['responses'].append(resp)
                click.echo(json.dumps(resp_list))
                return

            with click.progressbar(versions, label='Deleting Versions') as bar:
                for v in bar:
                    resp = self.util.cli_request('DELETE',
                        self.util.build_url('{app}/iot/v1/containers/{widget_id}/versions/{version_id}', {'widget_id': widget_id, 'version_id': v['id']}))

            self.util.print_table(versions, format)



"""
    This behavior file contains the logic for a subset of commands. Logic specific to
    commands should be implamented in corresponding behavior files.
"""

from ast import literal_eval
from pathlib import Path
from cli import pyke, behavior
import click
import time
import json
import sys
import os

class MicroserviceVersionBehavior:
    def __init__(self, profile=None, microservice=None, format=None, filter=None, page=None, pagesize=None, json=False):
        self.util = pyke.Util(profile=profile)
        self.object_type = 'microservice'
        self.json = json
        self.profile = profile
        self.microservice = microservice
        self.format = format
        self.filter = filter
        self.page = page
        self.pagesize = pagesize

    def list(self, microservice):
        microservice_id = self.util.lookup_object_id(self.object_type, microservice)
        data = {'filter': self.filter, 'page': self.page, 'pagesize': self.pagesize}

        handlers = {
          'actualState': lambda x: click.style(x, fg='red') if x not in ['running'] else click.style(x, fg='green')
        }

        resp = self.util.cli_request('GET',
            self.util.build_url('{app}/iot/v1/containers/{microservice_id}/versions?page={page}&pagesize={pagesize}&{filter}',
            {'microservice_id': microservice_id, **data}))

        if self.json:
            click.echo(json.dumps(resp))
            return

        self.util.print_table(resp['payload']['data'], self.format, handlers=handlers)
        self.util.print_record_count(resp)

    def start_stop(self, action, microservice, version):
        microservice_id = self.util.lookup_object_id(self.object_type, microservice)
        version_data = None

        try:
            version_id = int(version)
        except:
            major, minor, patch = self.util.parse_version_number(version)

            version_data = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/containers/{microservice_id}/versions?major={major}&minor={minor}&patch={patch}',\
                { 'microservice_id': microservice_id, 'major': major, 'minor': minor, 'patch': patch }))['payload']['data']

            if version_data is None or len(version_data) < 1:
                raise click.ClickException(click.style("Version not found", fg='red'))
            else:
                version_id = version_data[0].get('id')
                current_state = version_data[0].get('actualState')

        if version_data is None:
            version_data = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/containers/{microservice_id}/versions/{version_id}',\
                {'microservice_id': microservice_id, 'version_id': version_id} ))['payload']['data']

            if version_data is None or len(version_data) < 1:
                raise click.ClickException(click.style("Version not found", fg='red'))

            current_state = version_data.get('actualState')

        if current_state not in ['stopped', 'running', 'error']:
            raise click.ClickException(click.style("To start or stop a version it must have a 'stopped', 'running' or 'error' state. The current state of this version is {}. \
                A version goes through several states after being uploaded to Lumavate. It must finish this process before this command can be run. \
                To check its current state, run 'luma microservice-version ls'.".format(current_state), fg='yellow'))

        if action == 'stop' and current_state != 'running':
            raise click.ClickException(click.style("This version is not running", fg='yellow'))
        if action == 'start' and current_state == 'running':
            raise click.ClickException(click.style("This version is already running", fg='yellow'))

        titles = {
            'id': 'id',
            'actualState': 'Current State',
            'versionNumber': 'Version #',
            'createdAt': 'Created At',
            'createdBy': 'Created By'
        }

        data = {
            "action": action,
            "containerId": microservice_id,
            "id": version_id
        }

        resp = self.util.cli_request('POST', self.util.build_url('{app}/iot/v1/containers/{microservice_id}/versions/{version_id}/{action}',\
            {'microservice_id': microservice_id, 'version_id': version_id, 'action': action}), data=data)

        if self.json:
            click.echo(json.dumps(resp))
            return

        status_id = resp['payload']['data']['results']['statusId']
        progress = 0
        error = None
        if action == 'start':
            bar_name = "Starting microservice"
        else:
            bar_name = "Stopping microservice"

        with click.progressbar(length=100, label=bar_name) as bar:
            while progress < 100 and error is None:
                status_resp = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/statuses/{status_id}', {'status_id': status_id}))
                progress = status_resp['payload']['data']['overallPercent']
                error = status_resp['payload']['data']['errorMessage']
                bar.update(progress)

        click.echo(' ')
        ver_resp = self.util.cli_request('GET',
            self.util.build_url('{app}/iot/v1/containers/{microservice_id}/versions/{version_id}', {'microservice_id': microservice_id, 'version_id': version_id}))

        handlers = {
          'actualState': lambda x: click.style(x, fg='red') if x not in ['running'] else click.style(x, fg='green')
        }

        self.util.print_table([ver_resp['payload']['data']], self.format, handlers=handlers, titles=titles)

    def run(self, microservice, version, command):
        microservice_id = self.util.lookup_object_id(self.object_type, microservice)
        version_id = self.util.get_version_id(self.object_type, microservice_id, version)

        data = {
            "command": command
        }

        resp = self.util.cli_request('POST', self.util.build_url('{app}/iot/v1/containers/{microservice_id}/versions/{version_id}/exec',\
            {'microservice_id': microservice_id, 'version_id': version_id}), json=data)

        if self.json:
            click.echo(json.dumps(resp))
            return

        status_id = resp['payload']['data']['statusId']
        progress = 0
        error = None
        with click.progressbar(length=100, label='Executing command') as bar:
            while progress < 100:
                status_resp = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/statuses/{status_id}', {'status_id': status_id}))
                progress = status_resp['payload']['data']['overallPercent']
                bar.update(progress)
                if status_resp['payload']['data']['errorMessage'] is not None:
                    click.echo('There was an error running the command')
                    raise click.ClickException('Status: {}'.format(status_resp['payload']['data']))

        click.echo(' ')
        click.echo('Command Output: ')
        click.echo(' ')
        try:
            status_resp = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/statuses/{status_id}', {'status_id': status_id}))['payload']['data']
            summary = status_resp['summary']['results']
            if 'output' in summary[0].keys():
                if isinstance(summary[0]['output'], list):
                    for x in summary[0]['output']:
                        click.echo(x)
                else:
                    click.echo(summary)

        except:
            click.echo(status_resp)

    def logs(self, microservice, version):
        microservice_id = self.util.lookup_object_id(self.object_type, microservice)
        version_id = self.util.get_version_id(self.object_type, microservice_id, version)

        resp = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/containers/{microservice_id}/versions/{version_id}/logs',\
            {'microservice_id': microservice_id, 'version_id': version_id}))

        if self.json:
            click.echo(json.dumps(resp))
            return

        for x in resp['payload']['data']:
            click.echo(x)

    def update(self, microservice, version, label, format):
        microservice_id = self.util.lookup_object_id(self.object_type, microservice)
        version_id = self.util.get_version_id(self.object_type, microservice_id, version)

        post_data = {
          'microserviceId': microservice_id,
          'id': version_id,
          'label': label
        }

        resp = self.util.cli_request('PUT',\
          self.util.build_url('{app}/iot/v1/containers/{microservice_id}/versions/{version_id}',
          {'microservice_id': microservice_id, 'version_id': version_id}), data=json.dumps(post_data))

        if self.json:
            click.echo(json.dumps(resp))
            return

        self.util.print_table([resp['payload']['data']], format)

    def delete(self, microservice, version, version_mask, format):
        if version is None and version_mask is None:
            version = str(input("Version: "))
            if version is None or version == '':
                raise click.BadParameter(click.style('You must provide either a --version or a --version-mask', fg='red'))

        microservice_id = self.util.lookup_object_id(self.object_type, microservice)

        if version is not None:
            version_id = self.util.get_version_id(self.object_type, microservice_id, version)

            resp = self.util.cli_request('DELETE',\
              self.util.build_url('{app}/iot/v1/containers/{microservice_id}/versions/{version_id}', {'microservice_id': microservice_id, 'version_id': version_id}))

            if self.json:
                click.echo(json.dumps(resp))
                return

            self.util.print_table([resp['payload']['data']], format)

        else:
            versions = self.util.get_versions_from_mask(self.object_type, microservice_id, version_mask)

            if self.json:
                resp_list = {}
                resp_list['responses'] = []
                for v in versions:
                    resp = self.util.cli_request('DELETE',
                        self.util.build_url('{app}/iot/v1/containers/{microservice_id}/versions/{version_id}', {'microservice_id': microservice_id, 'version_id': v['id']}))
                    resp_list['responses'].append(resp)
                click.echo(json.dumps(resp_list))
                return

            with click.progressbar(versions, label='Deleting Versions') as bar:
                for v in bar:
                    resp = self.util.cli_request('DELETE',
                        self.util.build_url('{app}/iot/v1/containers/{microservice_id}/versions/{version_id}', {'microservice_id': microservice_id, 'version_id': v['id']}))

            self.util.print_table(versions, format)



"""
    These are the sub-commands for a top level object added to the CLI group in 'cli.py'.
    The commands and options are implemented here and the logic behind them resides in the corresponding behavior file.
"""

from cli import pyke, behavior
import click

@click.group(help='Experience Collection related commands.')
def experience_collection():
  pass

@click.command('ls', help="List the experience collections related to the selected profile.")
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--format', '-f', default='{id} {name} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
  {} and returns only that column. It is not compatible with --json flag.')
@pyke.list_options
@pyke.auth.profile_context_required
def list(**kwargs):
  behavior.ExperienceCollectionBehavior(**kwargs).list()

experience_collection.add_command(list)



import click

class CliException(click.ClickException):
  def __init__(self, message, **kwargs):
    if 'fg' not in kwargs:
      kwargs['fg'] = 'red'

    click.ClickException.__init__(self, click.style(message, **kwargs))



"""
    These sub-commands and their corresponding logic are both implamented in this file.
    TO DO: Break out the logic for these commands into a seperate behavior file.
"""

from pathlib import Path
from cli import pyke
import click
import json
import os

@click.group(help='An environment tells the CLI where to get valid tokens for calls to the platform. \
Environments are analogous to auth realms.')
def env():
    pass

@click.command('config')
@click.option('--env-name', prompt=True, help='The environment name')
@click.option('--app', prompt=True, help='The App URL')
@click.option('--token', prompt=True, help='The URL for auth tokens')
@click.option('--audience', prompt=True)
@click.option('--client-id', prompt=True)
@click.option('--client-secret', prompt=True)
@click.option('--json', 'json_flag', is_flag=True, help='Return raw json from the platform.')
def config(app, token, audience, client_id, client_secret, env_name, json_flag):
    config_data = pyke.auth.load_config()

    env_data = {
      'app': app,
      'token': token,
      'audience': audience,
      'clientId': client_id,
      'clientSecret': client_secret
    }

    config_data['envs'][env_name] = env_data

    pyke.Util().login_without_context(env_name, config_data=config_data)

    pyke.auth.save_config(config_data)

    if json_flag:
        click.echo(json.dumps(config_data))
        return
    else:
        data = []
        for key in config_data['envs'].keys():
            config_data['envs'][key]['envName'] = str(key)
            data.append(config_data['envs'][key])

        pyke.Util().print_table(data, '{envName} {app} {audience} {token}')

@click.command('ls')
@click.option('--format', '-f', default='{envName} {app} {audience} {token}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--json', 'json_flag', is_flag=True, help='Return raw json.')
def list(format, json_flag):
    config = pyke.auth.load_config()

    data = []
    for key in config['envs'].keys():
        config['envs'][key]['envName'] = str(key)
        data.append(config['envs'][key])

    if json_flag:
        click.echo(json.dumps(config['envs']))
        return

    pyke.Util().print_table(data, format)

@click.command('rm')
@click.argument('env_name')
@click.option('--force', is_flag=True, help='Skip input and delete all profiles using the environment being deleted')
def delete(env_name, force):
    config_data = pyke.auth.load_config()
    env_data = config_data["envs"].get(env_name)

    del config_data["envs"][env_name]

    profiles = config_data['profiles']
    profiles_to_del = []
    for profile in profiles:
      if profiles[profile].get('env') == env_name:
        profiles_to_del.append(profile)

    if len(profiles_to_del) > 0:
      click.echo(click.style("The following profiles are using this environment. Would you like to delete these profiles?", fg='white'))
      click.echo(click.style(str(profiles_to_del), fg='yellow'))
      click.echo(' ')

      if not force:
        try:
          cascade = input("Yes/No: ")
          cascade = str(cascade).lower()
        except:
          raise click.ClickException(click.style("The CLI expects either a 'Yes' or a 'No' as input", fg='red'))

        truthy_input = ['y', 'yes', 'true']
        if cascade in truthy_input:
          for profile in profiles_to_del:
            del config_data['profiles'][profile]

          if len(profiles_to_del) > 0:
            click.echo(' ')
            click.echo(click.style("Deleted profiles: {}".format(profiles_to_del), fg='white'))
          else:
            click.echo(' ')
            click.echo(click.style("No associated profiles", fg='white'))

      else:
        for profile in profiles_to_del:
          del config_data['profiles'][profile]

        if len(profiles_to_del) > 0:
          click.echo(' ')
          click.echo(click.style("Deleted profiles: {}".format(profiles_to_del), fg='white'))
        else:
          click.echo(' ')
          click.echo(click.style("No associated profiles", fg='white'))

    pyke.auth.save_config(config_data)
    click.echo(click.style("Deleted Environment: {}".format(env_name), fg='white'))


env.add_command(config)
env.add_command(list)
env.add_command(delete)



"""
    These are the sub-commands for a top level object added to the CLI group in 'cli.py'.
    The commands and options are implemented here and the logic behind them resides in the corresponding behavior file.
"""

from cli import pyke, behavior
import click

@click.group('component-set-version', help="A component-set-version is where the platform looks for components when used in a widget. \
The version also contains other details like CSS includes.")
def component_set_version():
    pass

@click.command('ls')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--component-set', '-cs', prompt=True, help='The id or the urlRef of the component-set')
@click.option('--format', '-f', default='{id} {versionNumber} {directIncludes} {directCssIncludes} {label} {expand__experiences} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--filter', default='')
@click.option('--page', default=1)
@click.option('--pagesize', default=100)
@click.option('--json/--table', default=False, help='Return raw json from the platform.')
@pyke.auth.profile_context_required
def list(component_set, profile, format, json, **kwargs):
    behavior.ComponentSetVersionBehavior(profile=profile, format=format, json=json, **kwargs).list(component_set)

@click.command('components', help="Given a --component-set and a --version, this will return the raw JSON contained in a component-set-version")
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--component-set', '-cs', prompt=True, help='The id or the urlRef of the component-set the version is associated with')
@click.option('--version', '-v', prompt=True)
@click.option('--json/--table', default=False, help='Return raw json from the platform.')
@pyke.auth.profile_context_required
def components(component_set, profile, version, json):
    behavior.ComponentSetVersionBehavior(profile=profile, json=json).list_components(component_set, version)

@click.command('add', help="While adding a new component-set-version you must always upload a zipped file, even if you use the from-version option")
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--component-set', '-cs', prompt=True, help='The id or the urlRef of the component-set the version is added to')
@click.option('--component-set-file-path', '-path', prompt=True, type=click.Path(file_okay=True, dir_okay=False), help='Must be the path to a zipped file.')
@click.option('--from-version', '-fv', type=str, help="Take all values from an existing version and create a new version. \
Increment the major, minor or patch version by 1 with --major, --minor, --patch flags. Override any old values by passing them in as options. \
When using this option with component-sets you will always need to provide a new, zipped, component set file.")
@click.option('--version', '-v', type=str, default=None, help="The version number of the new version")
@click.option('--patch', 'version_bump', flag_value='patch', default=True)
@click.option('--minor', 'version_bump', flag_value='minor')
@click.option('--major', 'version_bump', flag_value='major')
@click.option('--css-includes', type=str, default=None, multiple=True)
@click.option('--direct-includes', type=str, default=None, multiple=True)
@click.option('--label', '-l', type=click.Choice(['prod', 'dev', 'old']))
@click.option('--format', '-f', default='{id} {versionNumber} {directIncludes} {directCssIncludes} {label} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--json/--table', default=False, help='Return raw json from the platform.')
@pyke.auth.profile_context_required
def create(component_set, profile, component_set_file_path, from_version, version,
    version_bump, css_includes, direct_includes, label, format, json):
    behavior.ComponentSetVersionBehavior(profile=profile, format=format, json=json)\
        .create(component_set, component_set_file_path, from_version, version,
            version_bump, css_includes, direct_includes, label)

@click.command()
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--component-set', '-cs', prompt=True, help='The id or the urlRef of the component-set the version is associated with')
@click.option('--version', '-v', prompt=True, help='The ID or the version-number of the version to update')
@click.option('--label', '-l', prompt=True, type=str)
@click.option('--format', '-f', default='{id} {versionNumber} {directIncludes} {directCssIncludes} {label} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--json/--table', default=False, help='Return raw json from the platform.')
@pyke.auth.profile_context_required
def update(component_set, version, label, profile, format, json):
    behavior.ComponentSetVersionBehavior(profile=profile, format=format, json=json).update(component_set, version, label)

@click.command('rm')
@click.option('--profile', '-p', prompt=True, help='The name of the profile to use as the context for the command.', autocompletion=pyke.auth.get_profile_names)
@click.option('--component-set', '-cs', prompt=True, help='The id or the urlRef of the component-set the version is associated with')
@click.option('--version-mask', '-vm', help="Delete versions with version numbers that match this version mask")
@click.option('--version', '-v', help='The version ID or the version-number')
@click.option('--format', '-f', default='{id} {versionNumber} {directIncludes} {directCssIncludes} {label} {createdAt}', help='The --format option takes the column name of the returned table wrapped in \
{} and returns only that column. It is not compatible with --json flag.')
@click.option('--json/--table', default=False, help='Return raw json from the platform.')
@pyke.auth.profile_context_required
def delete(component_set, version_mask, version, profile, format, json):
    behavior.ComponentSetVersionBehavior(profile=profile, format=format, json=json).delete(component_set, version, version_mask)

component_set_version.add_command(list)
component_set_version.add_command(create)
component_set_version.add_command(delete)
component_set_version.add_command(update)
component_set_version.add_command(components)



from datetime import datetime
from pathlib import Path
from cli import pyke
import subprocess
import platform
import click
import json
import os

verify_tls = os.environ.get('LUMA_CLI_VERIFY_TLS', None)
if verify_tls == 'false' or verify_tls == 'False':
  verify_tls = False
else:
  verify_tls = None

class ContainerVersionBehavior:
  def __init__(self, **kwargs):
    self.profile = kwargs.get('profile')
    self.util = pyke.Util(profile=self.profile)
    self.object_type = 'container'

    if self.profile is not None:
      self.context = pyke.auth.load_cache()

    self.container           = kwargs.get('container')
    self.port                = kwargs.get('port')
    self.editor_port         = kwargs.get('editor_port')
    self.is_editable         = kwargs.get('is_editable')
    self.container_file_path = kwargs.get('container_file_path')
    self.docker_image        = kwargs.get('docker_image')
    self.from_version        = kwargs.get('from_version')
    self.version             = kwargs.get('version')
    self.version_bump        = kwargs.get('version_bump')
    self.env                 = kwargs.get('env', {})
    self.label               = kwargs.get('label')
    self.format              = kwargs.get('format')
    self.json                = kwargs.get('json')

  def echo(self, msg, color='white'):
    if self.json:
      return

    click.echo(click.style(str(msg), fg=color))

  def raise_exc(self, msg, color='red'):
    raise click.ClickException(click.style(msg, fg=color))

  def docker_image_path(self, image_name):
    self.echo("Getting docker image:")
    cu = pyke.ContainerUtil()

    return cu.save_image(image_name)

  def resolve_env(self):
    pass

  def create_version_data(self, container_id):
    if not self.from_version:
      version = {}

      # Set label
      if not self.label:
        if self.is_editable:
          label = 'dev'
        else:
          label = input("Label: ")
          if label is None or label == '':
            self.raise_exc("If you don't create from an old version then you must provide a --label")

        version['label'] = label
      else:
        version['label'] = self.label

      # Set version number
      if self.version:
        major, minor, patch = self.util.parse_version_number(self.version)

        version['major'] = major
        version['minor'] = minor
        version['patch'] = patch

      else:
        version_number = input("Version Number: ")

        if not version:
          self.raise_exc("If you don't create from an old version then you must provide a --version-number in the format\
              '<major: int>.<minor: int>.<patch: int>'")

        major, minor, patch = self.util.parse_version_number(version_number)

        version['major'] = major
        version['minor'] = minor
        version['patch'] = patch

      # Set port
      if not self.port:
        port = int(input("Port: "))
        if not port:
          self.raise_exc("If you don't create from an old version then you must provide a --port for your new version")
        version['port'] = port
      else:
        version['port'] = self.port

      return version

    else:
      version = self.util.get_container_version(container_id, self.from_version)
      from_version_id = version.get('id')

      if self.version:
        major, minor, patch = self.util.parse_version_number(self.version)

        version['major'] = major
        version['minor'] = minor
        version['patch'] = patch

        return version

      else:
        if self.version_bump == 'patch':
          versions = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/containers/{container_id}/versions?major={major}&minor={minor}&sort=patch+desc',\
                { 'container_id': version['containerId'], 'major': version['major'], 'minor': version['minor'] }))['payload']['data']
        elif self.version_bump == 'minor':
          versions = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/containers/{container_id}/versions?major={major}&sort=minor+desc',\
                { 'container_id': version['containerId'], 'major': version['major'] }))['payload']['data']
        else:
          versions = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/containers/{container_id}/versions?sort=major+desc',\
                { 'container_id': version['containerId'] }))['payload']['data']

        if versions is not None and len(versions) > 0:
          latest_version = versions[0]

        version[self.version_bump] = latest_version[self.version_bump] + 1
        if self.version_bump in ['major', 'minor']:
          version['patch'] = 0
        if self.version_bump == 'major':
          version['minor'] = 0

        return version

  def create(self, container_type):
    container = self.util.get_container(self.container)
    c_type = container.get('type')
    if c_type != container_type:
      self.raise_exc("The {} container was not found. (Id || UrlRef) - '{}'".format(container_type, self.container))

    container_id = container.get('id')

    if self.docker_image:
      container_file_path = self.docker_image_path(self.docker_image)
    else:
      container_file_path = self.container_file_path

    if not container_file_path and not self.from_version:
      image_name = input("Name of Docker Image: ")
      if not image_name:
        self.raise_exc("If you don't create from an old version then you must provide --container-file-path or --docker-image")

      container_file_path = self.docker_image_path(image_name)

    version = self.create_version_data(container_id)
    # Get version data

    if container_file_path:
      self.echo("Uploading image to Lumavate: ")
      self.echo('Image Size: {}'.format(self.util.get_size(container_file_path)))

      version['ephemeralKey'] = self.util.upload_ephemeral(container_file_path, 'application/gz')

    version['platformVersion'] = 'v2'
    if 'env' not in version.keys():
      version['env'] = {}

    for var in self.env:
      try:
        env_dict = json.loads(var)
        version['env'] = { **version['env'], **env_dict}
      except Exception as e:
        print("Could not serialize {}".format(var))
        print(e)

    if self.label:
      version['label'] = self.label

    if self.port:
      version['port'] = self.port

    if self.is_editable:
      version['isEditable'] = True
      version['label'] = 'dev'
      version['editorPort'] = 5001

    if self.editor_port:
      version['editorPort'] = self.editor_port

    version['instanceCount'] = 1

    resp = self.upload_version_json(version, container_id)

    handlers = {
          'actualState': lambda x: click.style(x, fg='red') if x not in ['running'] else click.style(x, fg='green')
          }

    self.util.print_table([resp['payload']['data']], self.format, handlers=handlers)

    if self.docker_image:
      try:
        os.remove(container_file_path)
      except:
        self.echo('Failed to delete zipped docker image...')

  def upload_version_json(self, version_data, container_id):
    resp = self.util.cli_request('POST', self.util.build_url('{app}/iot/v1/containers/{container_id}/versions',\
        {'container_id': container_id}), data=json.dumps(version_data))

    if self.json:
      return json.dumps(resp)

    # TO DO: Add progress bar after create for loading/validating container
    status_id = resp.get('payload', {}).get('data', {}).get('results', {}).get('statusId', {})
    if not status_id:
      return resp

    status_resp = self.util.show_progress(status_id, label='Uploading Service')

    error_msg = status_resp.get('payload', {}).get('data', {}).get('errorMessage')
    if error_msg is not None:
      self.raise_exc('Upload failed. {}'.format(error_msg))

    return resp

  def force_update (self):
    container = self.util.get_container(self.container)
    container_id = container.get('id')

    version = self.util.get_container_version(container_id, self.version)
    version_id = version.get('id')

    req_data = {'force': True}
    resp = self.util.cli_request('PUT',
        self.util.build_url('{app}/iot/v1/containers/{container_id}/versions/{version_id}',
        {'container_id': container_id, 'version_id': version_id}), json=req_data)

    click.echo(resp)

  def download_app_zip(self, container, version, path):
    home = str(Path.home())
    if path is None:
      path = home
    else:
      path = '{}/{}'.format(home, path)

    container_id = self.util.lookup_object_id(self.object_type, container)

    try:
      container = self.util.cli_request('GET',
          self.util.build_url('{app}/iot/v1/containers?id={id}', {'id': container_id} ))['payload']['data']
    except:
      raise click.ClickException('Container not found')

    url_ref = container[0].get('urlRef')
    version_id = self.util.get_version_id(self.object_type, container_id, version)

    resp = self.util.cli_request('POST',
        self.util.build_url('{app}/iot/v1/containers/{container_id}/versions/{version_id}/token',
          {'container_id': container_id, 'version_id': version_id}), json={'type': 'editor'})

    token = resp.get('payload', {}).get('data', {}).get('token')

    service_url = self.util.context.get('experienceCloudUri')

    # TODO: Dynamically get integration cloud
    editor_url = '{}/ic/{}/luma-editor/download/application.zip'.format(service_url, url_ref)

    time = datetime.now().microsecond

    if platform.system() == 'Windows':
      zip_path = '{}\\application.{}.zip'.format(path, time)
    else:
      zip_path = '{}/application.{}.zip'.format(path, time)

    command_list = 'curl -L -f --create-dirs --output {} {} -H '.format(zip_path, editor_url).split()
    command_list.append("Authorization: Bearer {}".format(token))
    if verify_tls is False:
      command_list.extend(['--insecure', '--proxy-insecure'])

    subprocess.run(command_list)
    click.echo("File Location: {}".format(zip_path))

  def tail_logs(self, container, version, tail_number=None):
    container = self.util.get_container(container)
    container_id = container.get('id')

    version = self.util.get_container_version(container_id, version)
    version_id = version.get('id')

    url_ref = container.get('urlRef')

    editorPort = version.get('editorPort')
    if not editorPort:
      return self.old_logs(container_id, version_id)

    resp = self.util.cli_request('POST',
        self.util.build_url('{app}/iot/v1/containers/{container_id}/versions/{version_id}/token',
          {'container_id': container_id, 'version_id': version_id}), json={'type': 'editor'})

    token = resp.get('payload', {}).get('data', {}).get('token')
    service_url = self.util.context.get('experienceCloudUri')

    if not tail_number:
      tail_number = 100

    # TODO: Dynamically get integration cloud
    editor_url = '{}/ic/{}/luma-editor/logs?tail={}'.format(service_url, url_ref, tail_number)

    command_list = 'curl {} -H '.format(editor_url).split()
    command_list.append("Authorization: Bearer {}".format(token))
    if verify_tls is False:
      command_list.extend(['--insecure', '--proxy-insecure'])

    subprocess.run(command_list)

  def old_logs(self, container_id, version_id):
    resp = self.util.cli_request('GET', self.util.build_url('{app}/iot/v1/containers/{container_id}/versions/{version_id}/logs',\
      {'container_id': container_id, 'version_id': version_id}))

    for x in resp['payload']['data']:
      click.echo(x)
